@@ -33,6 +33,7 @@
 #include "providers/backend.h"
 #include "util/crypto/sss_crypto.h"
 #include "util/cert.h"
+#include "util/sss_chain_id.h"
 #include "p11_child/p11_child.h"
 
 static const char *op_mode_str(enum op_mode mode)
@@ -147,7 +148,7 @@ int main(int argc, const char *argv[])
     poptContext pc;
     int debug_fd = -1;
     const char *opt_logger = NULL;
-    errno_t ret;
+    errno_t ret = 0;
     TALLOC_CTX *main_ctx = NULL;
     enum op_mode mode = OP_NONE;
     enum pin_mode pin_mode = PIN_NONE;
@@ -161,22 +162,15 @@ int main(int argc, const char *argv[])
     char *key_id = NULL;
     char *label = NULL;
     char *cert_b64 = NULL;
+    uint64_t chain_id = 0;
     bool wait_for_card = false;
     char *uri = NULL;
 
     struct poptOption long_options[] = {
         POPT_AUTOHELP
-        {"debug-level", 'd', POPT_ARG_INT, &debug_level, 0,
-         _("Debug level"), NULL},
-        {"debug-timestamps", 0, POPT_ARG_INT, &debug_timestamps, 0,
-         _("Add debug timestamps"), NULL},
-        {"debug-microseconds", 0, POPT_ARG_INT, &debug_microseconds, 0,
-         _("Show timestamps with microseconds"), NULL},
+        SSSD_DEBUG_OPTS
         {"debug-fd", 0, POPT_ARG_INT, &debug_fd, 0,
          _("An open file descriptor for the debug logs"), NULL},
-        {"debug-to-stderr", 0, POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN,
-         &debug_to_stderr, 0,
-         _("Send the debug output to stderr directly."), NULL },
         SSSD_LOGGER_OPTS
         {"auth", 0, POPT_ARG_NONE, NULL, 'a', _("Run in auth mode"), NULL},
         {"pre", 0, POPT_ARG_NONE, NULL, 'p', _("Run in pre-auth mode"), NULL},
@@ -202,6 +196,8 @@ int main(int argc, const char *argv[])
          _("certificate to verify, base64 encoded"), NULL},
         {"uri", 0, POPT_ARG_STRING, &uri, 0,
          _("PKCS#11 URI to restrict selection"), NULL},
+        {"chain-id", 0, POPT_ARG_LONG, &chain_id,
+         0, _("Tevent chain ID used for logging purposes"), NULL},
         POPT_TABLEEND
     };
 
@@ -306,23 +302,26 @@ int main(int argc, const char *argv[])
 
     poptFreeContext(pc);
 
-    DEBUG_INIT(debug_level);
-
     debug_prg_name = talloc_asprintf(NULL, "p11_child[%d]", getpid());
     if (debug_prg_name == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_asprintf failed.\n");
-        goto fail;
+        ERROR("talloc_asprintf failed.\n");
+        ret = ENOMEM;
+        goto done;
     }
 
     if (debug_fd != -1) {
+        opt_logger = sss_logger_str[FILES_LOGGER];
         ret = set_debug_file_from_fd(debug_fd);
         if (ret != EOK) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "set_debug_file_from_fd failed.\n");
+            opt_logger = sss_logger_str[STDERR_LOGGER];
+            ERROR("set_debug_file_from_fd failed.\n");
         }
-        opt_logger = sss_logger_str[FILES_LOGGER];
     }
 
-    sss_set_logger(opt_logger);
+    sss_chain_id_set_format(DEBUG_CHAIN_ID_FMT_CID);
+    sss_chain_id_set(chain_id);
+
+    DEBUG_INIT(debug_level, opt_logger);
 
     DEBUG(SSSDBG_TRACE_FUNC, "p11_child started.\n");
 
@@ -340,7 +339,8 @@ int main(int argc, const char *argv[])
     if (main_ctx == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new failed.\n");
         talloc_free(discard_const(debug_prg_name));
-        goto fail;
+        ret = ENOMEM;
+        goto done;
     }
     talloc_steal(main_ctx, debug_prg_name);
 
@@ -349,14 +349,16 @@ int main(int argc, const char *argv[])
                                 || key_id == NULL)) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               "--module_name, --token_name and --key_id must be given for "
-              "authentication");
-        goto fail;
+              "authentication\n");
+        ret = EINVAL;
+        goto done;
     }
 
     ret = parse_cert_verify_opts(main_ctx, verify_opts, &cert_verify_opts);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Failed to parse verify option.\n");
-        goto fail;
+        ret = EINVAL;
+        goto done;
     }
 
     if (mode == OP_VERIFIY && !cert_verify_opts->do_verification) {
@@ -369,27 +371,27 @@ int main(int argc, const char *argv[])
         ret = p11c_recv_data(main_ctx, STDIN_FILENO, &pin);
         if (ret != EOK) {
             DEBUG(SSSDBG_FATAL_FAILURE, "Failed to read PIN.\n");
-            goto fail;
+            ret = EINVAL;
+            goto done;
         }
     }
 
     ret = do_work(main_ctx, mode, ca_db, cert_verify_opts, wait_for_card,
                   cert_b64, pin, module_name, token_name, key_id, label, uri,
                   &multi);
-    if (ret != 0) {
-        DEBUG(SSSDBG_OP_FAILURE, "do_work failed.\n");
-        goto fail;
-    }
 
-    if (multi != NULL) {
-        fprintf(stdout, "%s", multi);
-    }
+done:
+    fprintf(stdout, "%d\n%s", ret, multi ? multi : "");
 
     talloc_free(main_ctx);
-    return EXIT_SUCCESS;
-fail:
-    DEBUG(SSSDBG_CRIT_FAILURE, "p11_child failed!\n");
-    close(STDOUT_FILENO);
-    talloc_free(main_ctx);
-    return EXIT_FAILURE;
+
+    if (ret == EOK) {
+        return EXIT_SUCCESS;
+    } else if (ret == ERR_CA_DB_NOT_FOUND) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "p11_child failed - CA DB not found\n");
+        return CA_DB_NOT_FOUND_EXIT_CODE;
+    } else {
+        DEBUG(SSSDBG_CRIT_FAILURE, "p11_child failed (%d)\n", ret);
+        return EXIT_FAILURE;
+    }
 }
