@@ -504,7 +504,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     struct ldb_message_element *el;
     struct sysdb_attrs *group_attrs;
     const char *group_name = NULL;
-    gid_t gid;
+    gid_t gid = 0;
     errno_t ret;
     char *usn_value = NULL;
     TALLOC_CTX *tmpctx = NULL;
@@ -2004,7 +2004,7 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
     }
 
     if (state->no_members) {
-        ret = sysdb_attrs_primary_fqdn_list(state->dom, state,
+        ret = sdap_get_primary_fqdn_list(state->dom, state,
                                 state->groups, state->count,
                                 state->opts->group_map[SDAP_AT_GROUP_NAME].name,
                                 &sysdb_groupnamelist);
