@@ -38,7 +38,6 @@ from util import unindent
 from sssd_nss import NssReturnCode
 from sssd_passwd import call_sssd_getpwnam, call_sssd_getpwuid
 from sssd_group import call_sssd_getgrnam, call_sssd_getgrgid
-from files_ops import passwd_ops_setup, group_ops_setup
 
 LDAP_BASE_DN = "dc=example,dc=com"
 INTERACTIVE_TIMEOUT = 4
@@ -54,7 +53,7 @@ def ds_inst(request):
 
     try:
         ds_inst.setup()
-    except:
+    except Exception:
         ds_inst.teardown()
         raise
     request.addfinalizer(ds_inst.teardown)
@@ -81,8 +80,8 @@ def cleanup_ldap_entries(ldap_conn, ent_list=None):
     """Remove LDAP entries added by create_ldap_entries"""
     if ent_list is None:
         for ou in ("Users", "Groups", "Netgroups", "Services", "Policies"):
-            for entry in ldap_conn.search_s("ou=" + ou + "," +
-                                            ldap_conn.ds_inst.base_dn,
+            for entry in ldap_conn.search_s(f"ou={ou},"
+                                            f"{ldap_conn.ds_inst.base_dn}",
                                             ldap.SCOPE_ONELEVEL,
                                             attrlist=[]):
                 ldap_conn.delete_s(entry[0])
@@ -210,10 +209,10 @@ def cleanup_sssd_process():
         while True:
             try:
                 os.kill(pid, signal.SIGCONT)
-            except:
+            except OSError:
                 break
             time.sleep(1)
-    except:
+    except OSError:
         pass
     for path in os.listdir(config.DB_PATH):
         os.unlink(config.DB_PATH + "/" + path)
@@ -414,8 +413,8 @@ def test_sanity_rfc2307_bis(ldap_conn, sanity_rfc2307_bis):
 
 
 def test_member_with_different_cases_rfc2307_bis(
-                                      ldap_conn,
-                                      member_with_different_cases_rfc2307_bis):
+        ldap_conn,
+        member_with_different_cases_rfc2307_bis):
     """
     Regression test for https://bugzilla.redhat.com/show_bug.cgi?id=1817122
     Make sure that group members are added properly to the group even if the
@@ -822,7 +821,7 @@ def test_user_2307bis_nested_groups(ldap_conn,
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(expected_gids)])
-        )
+    )
 
 
 def test_special_characters_in_names(ldap_conn, sanity_rfc2307):
@@ -1200,7 +1199,7 @@ def test_nss_filters(ldap_conn, sanity_nss_filter):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user_with_group_ids)])
-        )
+    )
 
 
 @pytest.fixture
@@ -1334,7 +1333,7 @@ def test_ldap_auto_private_groups_direct(ldap_conn, mpg_setup):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user1_expected_gids)])
-        )
+    )
 
     # Request user2's private group by GID without resolving the user first.
     # This must trigger user resolution through by-GID resolution, since the
@@ -1350,7 +1349,7 @@ def test_ldap_auto_private_groups_direct(ldap_conn, mpg_setup):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user1_expected_gids)])
-        )
+    )
 
     # Request user3's private group by name without resolving the user first
     # This must trigger user resolution through by-name resolution, since the
@@ -1428,7 +1427,7 @@ def test_ldap_auto_private_groups_conflict(ldap_conn, mpg_setup_conflict):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user3_expected_gids)])
-        )
+    )
     # Make sure the private group is resolvable by name and by GID
     ent.assert_group_by_gid(1003, dict(name="user3", mem=ent.contains_only()))
     ent.assert_group_by_name("user3", dict(gid=1003, mem=ent.contains_only()))
@@ -1487,7 +1486,7 @@ def test_ldap_auto_private_groups_direct_no_gid(ldap_conn, mpg_setup_no_gid):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user1_expected_gids)])
-        )
+    )
 
 
 @pytest.fixture
@@ -1579,7 +1578,7 @@ def test_ldap_auto_private_groups_hybrid_direct(ldap_conn, mpg_setup_hybrid):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user_with_group_ids)])
-        )
+    )
 
     # On the other hand, if the gidNumber is the same as UID, SSSD should
     # just autogenerate the private group on its own
@@ -1598,37 +1597,33 @@ def test_ldap_auto_private_groups_hybrid_direct(ldap_conn, mpg_setup_hybrid):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user_without_group_ids)])
-        )
+    )
 
     ent.assert_passwd_by_name("user_with_unresolvable_gid",
                               dict(name="user_with_unresolvable_gid",
                                    uid=1003, gid=2003))
     unresolvable_group_ids = [2003, 10030, 10031]
-    (res, errno, gids) = sssd_id.call_sssd_initgroups(
-                                        "user_with_unresolvable_gid",
-                                        2003)
+    (res, errno, gids) = sssd_id.call_sssd_initgroups("user_with_unresolvable_gid", 2003)
     assert res == sssd_id.NssReturnCode.SUCCESS
 
     assert sorted(gids) == sorted(unresolvable_group_ids), \
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(unresolvable_group_ids)])
-        )
+    )
 
     ent.assert_passwd_by_name("user_with_real_group",
                               dict(name="user_with_real_group",
                                    uid=1004, gid=1004))
     with_real_group_ids = [1004, 10040, 10041]
-    (res, errno, gids) = sssd_id.call_sssd_initgroups(
-                                        "user_with_real_group",
-                                        1004)
+    (res, errno, gids) = sssd_id.call_sssd_initgroups("user_with_real_group", 1004)
     assert res == sssd_id.NssReturnCode.SUCCESS
 
     assert sorted(gids) == sorted(with_real_group_ids), \
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(with_real_group_ids)])
-        )
+    )
 
 
 def test_ldap_auto_private_groups_hybrid_priv_group_byname(ldap_conn,
@@ -1728,7 +1723,7 @@ def test_ldap_auto_private_groups_hybrid_name_gid_identical(ldap_conn,
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user_without_group_ids)])
-        )
+    )
     ent.assert_group_by_gid(1006,
                             dict(name="u_g_same_name",
                                  mem=ent.contains_only('user_in_pvt_group')))
@@ -1747,7 +1742,7 @@ def test_ldap_auto_private_groups_hybrid_initgr(ldap_conn, mpg_setup_hybrid):
         "result: %s\n expected %s" % (
             ", ".join(["%s" % s for s in sorted(gids)]),
             ", ".join(["%s" % s for s in sorted(user_without_group_ids)])
-        )
+    )
 
     ent.assert_group_by_gid(1004,
                             dict(name="user_with_real_group_pvt",
@@ -2045,8 +2040,7 @@ def test_local_negative_timeout_disabled(ldap_conn,
     cleanup_ldap_entries(ldap_conn, ent_list)
 
 
-@pytest.fixture
-def users_with_email_setup(request, ldap_conn):
+def users_with_email_setup(request, ldap_conn, cache_first):
     ent_list = ldap_ent.List(ldap_conn.ds_inst.base_dn)
     ent_list.add_user("user1", 1001, 2001, mail="user1.email@LDAP")
 
@@ -2059,27 +2053,35 @@ def users_with_email_setup(request, ldap_conn):
     create_ldap_fixture(request, ldap_conn, ent_list)
 
     conf = format_basic_conf(ldap_conn, SCHEMA_RFC2307_BIS)
+
+    conf += unindent("""
+        [nss]
+        cache_first = {0}
+    """).format(str(cache_first))
+
     create_conf_fixture(request, conf)
     create_sssd_fixture(request)
-    return None
 
 
-def test_lookup_by_email(ldap_conn, users_with_email_setup):
+@pytest.mark.parametrize('cache_first', [True, False])
+def test_lookup_by_email(request, ldap_conn, cache_first):
     """
     Test the simple case of looking up a user by e-mail
     """
+    users_with_email_setup(request, ldap_conn, cache_first)
     ent.assert_passwd_by_name("user1.email@LDAP",
                               dict(name="user1", uid=1001, gid=2001))
 
 
-def test_conflicting_mail_addresses_and_fqdn(ldap_conn,
-                                             users_with_email_setup):
+@pytest.mark.parametrize('cache_first', [True, False])
+def test_conflicting_mail_addresses_and_fqdn(request, ldap_conn, cache_first):
     """
     Test that we handle the case where one user's mail address is the
     same as another user's FQDN
 
     This is a regression test for https://github.com/SSSD/sssd/issues/4630
     """
+    users_with_email_setup(request, ldap_conn, cache_first)
     # With #3607 unfixed, these two lookups would prime the cache with
     # nameAlias: emailuser@LDAP for both entries..
     ent.assert_passwd_by_name("emailuser@LDAP",
@@ -2095,12 +2097,13 @@ def test_conflicting_mail_addresses_and_fqdn(ldap_conn,
                               dict(name="emailuser2", uid=1003, gid=2003))
 
 
-def test_conflicting_mail_addresses(ldap_conn,
-                                    users_with_email_setup):
+@pytest.mark.parametrize('cache_first', [True, False])
+def test_conflicting_mail_addresses(request, ldap_conn, cache_first):
     """
     Negative test: looking up a user by e-mail which belongs to more than
     one account fails in the back end.
     """
+    users_with_email_setup(request, ldap_conn, cache_first)
     with pytest.raises(KeyError):
         pwd.getpwnam("userxy@LDAP")
 
