@@ -239,6 +239,12 @@ struct sss_names_ctx {
     sss_regexp_t *re;
 };
 
+#define SSS_DEFAULT_RE "(?P<name>[^@]+)@?(?P<domain>[^@]*$)"
+
+#define SSS_IPA_AD_DEFAULT_RE "(((?P<domain>[^\\\\]+)\\\\(?P<name>.+$))|" \
+                              "((?P<name>.+)@(?P<domain>[^@]+$))|" \
+                              "(^(?P<name>[^@\\\\]+)$))"
+
 /* initialize sss_names_ctx directly from arguments */
 int sss_names_init_from_args(TALLOC_CTX *mem_ctx,
                              const char *re_pattern,
@@ -350,26 +356,6 @@ errno_t check_file(const char *filename,
                    uid_t uid, gid_t gid, mode_t mode, mode_t mask,
                    struct stat *caller_stat_buf, bool follow_symlink);
 
-/* check_fd()
- * Verify that an open file descriptor has certain permissions and/or
- * is of a certain file type. This function CANNOT detect symlinks,
- * as the file is already open and symlinks have been traversed. This
- * is the safer way to perform file checks and should be preferred
- * over check_file for nearly all situations.
- */
-errno_t check_fd(int fd, uid_t uid, gid_t gid,
-                 mode_t mode, mode_t mask,
-                 struct stat *caller_stat_buf);
-
-/* check_and_open_readonly()
- * Utility function to open a file and verify that it has certain
- * permissions and is of a certain file type. This function wraps
- * check_fd(), and is considered race-condition safe.
- */
-errno_t check_and_open_readonly(const char *filename, int *fd,
-                                uid_t uid, gid_t gid,
-                                mode_t mode, mode_t mask);
-
 /* from util.c */
 #define SSS_NO_LINKLOCAL 0x01
 #define SSS_NO_LOOPBACK 0x02
@@ -814,4 +800,24 @@ errno_t sss_getenv(TALLOC_CTX *mem_ctx,
                    const char *default_value,
                    char **_value);
 
+/* from sss_time.c */
+uint64_t get_start_time(void);
+
+const char *sss_format_time(uint64_t us);
+uint64_t get_spend_time_us(uint64_t st);
+
+/* from pac_utils.h */
+#define CHECK_PAC_NO_CHECK_STR "no_check"
+#define CHECK_PAC_PRESENT_STR "pac_present"
+#define CHECK_PAC_PRESENT (1 << 0)
+#define CHECK_PAC_CHECK_UPN_STR "check_upn"
+#define CHECK_PAC_CHECK_UPN (1 << 1)
+#define CHECK_PAC_UPN_DNS_INFO_PRESENT_STR "upn_dns_info_present"
+#define CHECK_PAC_UPN_DNS_INFO_PRESENT (1 << 2)
+#define CHECK_PAC_CHECK_UPN_DNS_INFO_EX_STR "check_upn_dns_info_ex"
+#define CHECK_PAC_CHECK_UPN_DNS_INFO_EX (1 << 3)
+#define CHECK_PAC_UPN_DNS_INFO_EX_PRESENT_STR "upn_dns_info_ex_present"
+#define CHECK_PAC_UPN_DNS_INFO_EX_PRESENT (1 << 4)
+
+errno_t get_pac_check_config(struct confdb_ctx *cdb, uint32_t *pac_check_opts);
 #endif /* __SSSD_UTIL_H__ */
